@@ -25,13 +25,11 @@
 #include "messages.h"
 #include "cache.hpp"
 #include "shared_cache.hpp"
-#include "canvas_base.h"
 #include "canvas_utils.h"
 #include "glz_decoded_image.h"
 #include "glz_decoder.h"
 #include "jpeg_decoder.h"
 #include "zlib_decoder.h"
-#include <map>
 
 enum CanvasType {
     CANVAS_TYPE_INVALID,
@@ -40,6 +38,102 @@ enum CanvasType {
     CANVAS_TYPE_GDI,
 };
 
+template <class T, int HASH_SIZE, class Base = EmptyBase>
+class CHash : public Base {
+public:
+    CHash()
+    {
+        memset(_hash, 0, sizeof(_hash));
+    }
+
+    ~CHash()
+    {
+    }
+
+    void add(uint32_t id, T* data)
+    {
+        Item** item = &_hash[key(id)];
+
+        while (*item) {
+            PANIC_ON((*item)->id == id);
+            item = &(*item)->next;
+        }
+        *item = new Item(id, data);
+    }
+
+    bool is_present(uint32_t id)
+    {
+        Item* item = _hash[key(id)];
+
+        for (;;) {
+            if (!item) {
+                return false;
+            }
+
+            if (item->id != id) {
+                item = item->next;
+                continue;
+            }
+
+            return true;
+        }
+    }
+
+    T* get(uint32_t id)
+    {
+        Item* item = _hash[key(id)];
+
+        for (;;) {
+            PANIC_ON(!item);
+
+            if (item->id != id) {
+                item = item->next;
+                continue;
+            }
+
+            return item->data;
+        }
+    }
+
+    void remove(uint32_t id)
+    {
+        Item** item = &_hash[key(id)];
+
+        while (*item) {
+            if ((*item)->id == id) {
+                Item *rm_item = *item;
+                *item = rm_item->next;
+                delete rm_item;
+                return;
+            }
+            item = &(*item)->next;
+        }
+        THROW("id %lu, not found", id);
+    }
+
+private:
+    inline uint32_t key(uint32_t id) {return id % HASH_SIZE;}
+
+private:
+    class Item {
+    public:
+        Item(uint32_t in_id, T* data)
+            : id (in_id)
+            , next (NULL)
+            , data (data) {}
+
+        ~Item()
+        {
+        }
+
+        uint64_t id;
+        Item* next;
+        T* data;
+    };
+
+    Item* _hash[HASH_SIZE];
+};
+
 class PixmapCacheTreat {
 public:
     static inline pixman_image_t *get(pixman_image_t *surf)
@@ -106,6 +200,44 @@ public:
     }
 };
 
+class SpiceImageSurfacesBase;
+
+typedef CHash<SpiceCanvas, 1024, SpiceImageSurfacesBase> CSurfaces;
+
+class SpiceImageSurfacesBase {
+public:
+    SpiceImageSurfaces base;
+
+    static void op_put(SpiceImageSurfaces *c, uint32_t surface_id, SpiceCanvas *surface)
+    {
+        CSurfaces* cache = reinterpret_cast<CSurfaces*>(c);
+        cache->add(surface_id, surface);
+    }
+
+    static SpiceCanvas* op_get(SpiceImageSurfaces *s, uint32_t surface_id)
+    {
+        CSurfaces* cache = reinterpret_cast<CSurfaces*>(s);
+        return cache->get(surface_id);
+    }
+
+    static void op_del(SpiceImageSurfaces *c, uint32_t surface_id)
+    {
+        CSurfaces* cache = reinterpret_cast<CSurfaces*>(c);
+        cache->remove(surface_id);
+    }
+
+    SpiceImageSurfacesBase()
+    {
+        static SpiceImageSurfacesOps cache_ops = {
+            op_get
+        };
+        base.ops = &cache_ops;
+    }
+};
+
+class Canvas;
+
+typedef CHash<Canvas, 1024, SpiceImageSurfacesBase> CCanvases;
 
 class CachedPalette {
 public:
@@ -267,20 +399,10 @@ public:
     }
 };
 
-class Canvas;
-
-typedef std::map<uint32_t, Canvas*> SurfacesCanvasesMap;
-
-class SurfacesCache: public SpiceImageSurfaces, public SurfacesCanvasesMap {
-public:
-    SurfacesCache();
-    bool exist(uint32_t surface_id);
-};
-
 class Canvas {
 public:
     Canvas(PixmapCache& bits_cache, PaletteCache& palette_cache,
-           GlzDecoderWindow &glz_decoder_window, SurfacesCache& csurfaces);
+           GlzDecoderWindow &glz_decoder_window, CSurfaces& csurfaces);
     virtual ~Canvas();
 
     virtual void copy_pixels(const QRegion& region, RedDrawable* dc,
@@ -320,7 +442,7 @@ protected:
 
     PixmapCache& pixmap_cache() { return _pixmap_cache;}
     PaletteCache& palette_cache() { return _palette_cache;}
-    SurfacesCache& surfaces_cache() { return _surfaces_cache;}
+    CSurfaces& csurfaces() { return _csurfaces; }
 
     GlzDecoder& glz_decoder() {return _glz_decoder;}
     JpegDecoder& jpeg_decoder() { return _jpeg_decoder;}
@@ -331,6 +453,7 @@ private:
 
 protected:
     SpiceCanvas* _canvas;
+    CSurfaces _surfaces;
 
 private:
     PixmapCache& _pixmap_cache;
@@ -343,7 +466,7 @@ private:
     JpegDecoder _jpeg_decoder;
     ZlibDecoder _zlib_decoder;
 
-    SurfacesCache& _surfaces_cache;
+    CSurfaces& _csurfaces;
 
     unsigned long _base;
     unsigned long _max;
