@@ -15,11 +15,14 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "common.h"
 #include "canvas.h"
 #include "red_pixmap.h"
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 #include "red_pixmap_gl.h"
 #endif
 #include "debug.h"
@@ -28,7 +31,7 @@
 #include "display_channel.h"
 #include "application.h"
 #include "screen.h"
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 #include "red_gl_canvas.h"
 #endif
 #include "red_sw_canvas.h"
@@ -365,7 +368,7 @@ void VideoStream::remove_dead_frames(uint32_t mm_time)
 void VideoStream::drop_one_frame()
 {
     ASSERT(MAX_VIDEO_FRAMES > 2 && (_frames_head - _frames_tail) == MAX_VIDEO_FRAMES);
-    int frame_index = _frames_head - _kill_mark++ % (MAX_VIDEO_FRAMES - 2) - 2;
+    unsigned frame_index = _frames_head - _kill_mark++ % (MAX_VIDEO_FRAMES - 2) - 2;
 
     free_frame(frame_index);
 
@@ -484,7 +487,7 @@ void StreamsTrigger::on_event()
     _channel.on_streams_trigger();
 }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 
 GLInterruptRecreate::GLInterruptRecreate(DisplayChannel& channel)
     : _channel (channel)
@@ -603,7 +606,7 @@ DisplayChannel::DisplayChannel(RedClient& client, uint32_t id,
     , _inputs_channel (NULL)
     , _active_streams (NULL)
     , _streams_trigger (*this)
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     , _gl_interrupt_recreate (*this)
 #endif
     , _interrupt_update (*this)
@@ -640,7 +643,7 @@ DisplayChannel::DisplayChannel(RedClient& client, uint32_t id,
     handler->set_handler(SPICE_MSG_DISPLAY_SURFACE_DESTROY, &DisplayChannel::handle_surface_destroy);
 
     get_process_loop().add_trigger(_streams_trigger);
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     get_process_loop().add_trigger(_gl_interrupt_recreate);
 #endif
     get_process_loop().add_trigger(_interrupt_update);
@@ -724,7 +727,7 @@ void DisplayChannel::copy_pixels(const QRegion& dest_region,
     canvas->copy_pixels(dest_region, NULL, &dest_pixmap);
 }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 void DisplayChannel::recreate_ogl_context_interrupt()
 {
     Canvas* canvas;
@@ -882,7 +885,7 @@ void DisplayChannel::set_capture_mode(bool on)
 
 void DisplayChannel::update_interrupt()
 {
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     Canvas *canvas;
 #endif
 
@@ -890,7 +893,7 @@ void DisplayChannel::update_interrupt()
         return;
     }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     canvas = surfaces_mngr.get_canvas(0);
     if (canvas->get_pixmap_type() == CANVAS_TYPE_GL) {
         ((GCanvas *)(canvas))->pre_gl_copy();
@@ -899,14 +902,14 @@ void DisplayChannel::update_interrupt()
 
     screen()->update();
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     if (canvas->get_pixmap_type() == CANVAS_TYPE_GL) {
         ((GCanvas *)(canvas))->post_gl_copy();
     }
 #endif
 }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 
 void DisplayChannel::pre_migrate()
 {
@@ -914,7 +917,7 @@ void DisplayChannel::pre_migrate()
 
 void DisplayChannel::post_migrate()
 {
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     if (surfaces_mngr.get_canvas(0)->get_pixmap_type() == CANVAS_TYPE_GL) {
         _gl_interrupt_recreate.trigger();
     }
@@ -1082,7 +1085,7 @@ bool DisplayChannel::create_sw_canvas(int surface_id, int width, int height, uin
     return true;
 }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
 bool DisplayChannel::create_ogl_canvas(int surface_id, int width, int height, uint32_t format,
                                        bool recreate, RenderType rendertype)
 {
@@ -1142,7 +1145,7 @@ void DisplayChannel::destroy_canvas(int surface_id)
 
     canvas = surfaces_mngr.get_canvas(surface_id);
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     if (canvas->get_pixmap_type() == CANVAS_TYPE_GL) {
         ((GCanvas *)(canvas))->touch_context();
     }
@@ -1157,12 +1160,12 @@ void DisplayChannel::destroy_canvas(int surface_id)
 void DisplayChannel::create_canvas(int surface_id, const std::vector<int>& canvas_types, int width,
                                    int height, uint32_t format)
 {
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     bool recreate = true;
 #endif
     unsigned int i;
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     if (screen()->need_recreate_context_gl()) {
         recreate = false;
     }
@@ -1175,7 +1178,7 @@ void DisplayChannel::create_canvas(int surface_id, const std::vector<int>& canva
         if (canvas_types[i] == CANVAS_OPTION_SW && create_sw_canvas(surface_id, width, height, format)) {
             break;
         }
-#ifdef USE_OGL
+#ifdef USE_OPENGL
         if (canvas_types[i] == CANVAS_OPTION_OGL_FBO && create_ogl_canvas(surface_id, width, height, format,
                                                                           recreate,
                                                                           RENDER_TYPE_FBO)) {
@@ -1407,7 +1410,7 @@ void DisplayChannel::handle_stream_destroy_all(RedPeer::InMessage* message)
 
 void DisplayChannel::create_primary_surface(int width, int height, uint32_t format)
 {
-#ifdef USE_OGL
+#ifdef USE_OPENGL
    Canvas *canvas;
 #endif
    _mark = false;
@@ -1426,7 +1429,7 @@ void DisplayChannel::create_primary_surface(int width, int height, uint32_t form
     _y_res = height;
     _format = format;
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     canvas = surfaces_mngr.get_canvas(0);
 
     if (canvas->get_pixmap_type() == CANVAS_TYPE_GL) {
@@ -1447,7 +1450,7 @@ void DisplayChannel::create_surface(int surface_id, int width, int height, uint3
         THROW("Create surface failed");
     }
 
-#ifdef USE_OGL
+#ifdef USE_OPENGL
     Canvas *canvas;
 
     canvas = surfaces_mngr.get_canvas(surface_id);
@@ -1461,7 +1464,7 @@ void DisplayChannel::create_surface(int surface_id, int width, int height, uint3
 void DisplayChannel::destroy_primary_surface()
 {
     if (screen()) {
-#ifdef USE_OGL
+#ifdef USE_OPENGL
         if (surfaces_mngr.is_present_canvas(0)) {
             Canvas *canvas;
 
