@@ -14,6 +14,9 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "common.h"
 #include "red_client.h"
@@ -169,37 +172,19 @@ PlaybackChannel::PlaybackChannel(RedClient& client, uint32_t id)
     set_capability(SPICE_PLAYBACK_CAP_CELT_0_5_1);
 }
 
-void PlaybackChannel::clear()
+PlaybackChannel::~PlaybackChannel(void)
 {
-    if (_wave_player) {
-        _playing = false;
-        _wave_player->stop();
-        delete _wave_player;
-        _wave_player = NULL;
-    }
-    _mode = SPICE_AUDIO_DATA_MODE_INVALID;
+    delete _wave_player;
 
     if (_celt_decoder) {
         celt051_decoder_destroy(_celt_decoder);
-        _celt_decoder = NULL;
     }
 
     if (_celt_mode) {
         celt051_mode_destroy(_celt_mode);
-        _celt_mode = NULL;
     }
 }
 
-void PlaybackChannel::on_disconnect()
-{
-    clear();
-}
-
-PlaybackChannel::~PlaybackChannel(void)
-{
-    clear();
-}
-
 bool PlaybackChannel::abort(void)
 {
     return (!_wave_player || _wave_player->abort()) && RedChannel::abort();
