@@ -15,6 +15,9 @@
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "common.h"
 #include "red_channel.h"
@@ -27,15 +30,6 @@
 #include "openssl/evp.h"
 #include "openssl/x509.h"
 
-void MigrationDisconnectSrcEvent::response(AbstractProcessLoop& events_loop)
-{
-    static_cast<RedChannel*>(events_loop.get_owner())->do_migration_disconnect_src();
-}
-
-void MigrationConnectTargetEvent::response(AbstractProcessLoop& events_loop)
-{
-    static_cast<RedChannel*>(events_loop.get_owner())->do_migration_connect_target();
-}
 
 RedChannelBase::RedChannelBase(uint8_t type, uint8_t id, const ChannelCaps& common_caps,
                                const ChannelCaps& caps)
@@ -51,7 +45,7 @@ RedChannelBase::~RedChannelBase()
 {
 }
 
-const char *spice_link_error_string(int err)
+static const char *spice_link_error_string(int err)
 {
     switch (err) {
         case SPICE_LINK_ERR_OK: return "no error";
@@ -125,7 +119,7 @@ void RedChannelBase::link(uint32_t connection_id, const std::string& password,
     send(buffer, p - buffer);
     delete [] buffer;
 
-    recive((uint8_t*)&header, sizeof(header));
+    receive((uint8_t*)&header, sizeof(header));
 
     if (header.magic != SPICE_MAGIC) {
         THROW_ERR(SPICEC_ERROR_CODE_CONNECT_FAILED, "bad magic");
@@ -142,7 +136,7 @@ void RedChannelBase::link(uint32_t connection_id, const std::string& password,
     _remote_minor = header.minor_version;
 
     AutoArray<uint8_t> reply_buf(new uint8_t[header.size]);
-    recive(reply_buf.get(), header.size);
+    receive(reply_buf.get(), header.size);
 
     reply = (SpiceLinkReply *)reply_buf.get();
 
@@ -206,7 +200,7 @@ void RedChannelBase::link(uint32_t connection_id, const std::string& password,
 
     BIO_free(bioKey);
 
-    recive((uint8_t*)&link_res, sizeof(link_res));
+    receive((uint8_t*)&link_res, sizeof(link_res));
     if (link_res != SPICE_LINK_ERR_OK) {
         int error_code = (link_res == SPICE_LINK_ERR_PERMISSION_DENIED) ?
                                 SPICEC_ERROR_CODE_CONNECT_FAILED : SPICEC_ERROR_CODE_CONNECT_FAILED;
@@ -295,20 +289,6 @@ bool RedChannelBase::test_capability(uint32_t cap)
     return test_capability(_remote_caps, cap);
 }
 
-void RedChannelBase::swap(RedChannelBase* other)
-{
-    int tmp_ver;
-
-    RedPeer::swap(other);
-    tmp_ver = _remote_major;
-    _remote_major = other->_remote_major;
-    other->_remote_major = tmp_ver;
-
-    tmp_ver = _remote_minor;
-    _remote_minor = other->_remote_minor;
-    other->_remote_minor = tmp_ver;
-}
-
 SendTrigger::SendTrigger(RedChannel& channel)
     : _channel (channel)
 {
@@ -383,10 +363,10 @@ void RedChannel::post_message(RedChannel::OutMessage* message)
     _send_trigger.trigger();
 }
 
-RedPeer::CompoundInMessage *RedChannel::recive()
+RedPeer::CompoundInMessage *RedChannel::receive()
 {
-    CompoundInMessage *message = RedChannelBase::recive();
-    on_message_recived();
+    CompoundInMessage *message = RedChannelBase::receive();
+    on_message_received();
     return message;
 }
 
@@ -460,63 +440,6 @@ void RedChannel::disconnect()
     _action_cond.notify_one();
 }
 
-void RedChannel::disconnect_migration_src()
-{
-    clear_outgoing_messages();
-
-    Lock lock(_action_lock);
-    if (_state == CONNECTING_STATE || _state == CONNECTED_STATE) {
-        AutoRef<MigrationDisconnectSrcEvent> migrate_event(new MigrationDisconnectSrcEvent());
-        _loop.push_event(*migrate_event);
-    }
-}
-
-void RedChannel::connect_migration_target()
-{
-    LOG_INFO("");
-    AutoRef<MigrationConnectTargetEvent> migrate_event(new MigrationConnectTargetEvent());
-    _loop.push_event(*migrate_event);
-}
-
-void RedChannel::do_migration_disconnect_src()
-{
-    if (_socket_in_loop) {
-        _socket_in_loop = false;
-        _loop.remove_socket(*this);
-    }
-
-    clear_outgoing_messages();
-    if (_outgoing_message) {
-        _outgoing_message->release();
-        _outgoing_message = NULL;
-    }
-    _incomming_header_pos = 0;
-    if (_incomming_message) {
-        _incomming_message->unref();
-        _incomming_message = NULL;
-    }
-
-    on_disconnect_mig_src();
-    get_client().migrate_channel(*this);
-    get_client().on_channel_disconnect_mig_src_completed(*this);
-}
-
-void RedChannel::do_migration_connect_target()
-{
-    LOG_INFO("");
-    ASSERT(get_client().get_protocol() != 0);
-    if (get_client().get_protocol() == 1) {
-        _marshallers = spice_message_marshallers_get1();
-    } else {
-        _marshallers = spice_message_marshallers_get();
-    }
-    _loop.add_socket(*this);
-    _socket_in_loop = true;
-    on_connect_mig_target();
-    set_state(CONNECTED_STATE);
-    on_event();
-}
-
 void RedChannel::clear_outgoing_messages()
 {
     Lock lock(_outgoing_lock);
@@ -670,7 +593,7 @@ void RedChannel::on_send_trigger()
     send_messages();
 }
 
-void RedChannel::on_message_recived()
+void RedChannel::on_message_received()
 {
     if (_message_ack_count && !--_message_ack_count) {
         post_message(new Message(SPICE_MSGC_ACK));
@@ -685,10 +608,10 @@ void RedChannel::on_message_complition(uint64_t serial)
     _sync_info.condition->notify_all();
 }
 
-void RedChannel::recive_messages()
+void RedChannel::receive_messages()
 {
     for (;;) {
-        uint32_t n = RedPeer::recive((uint8_t*)&_incomming_header, sizeof(SpiceDataHeader));
+        uint32_t n = RedPeer::receive((uint8_t*)&_incomming_header, sizeof(SpiceDataHeader));
         if (n != sizeof(SpiceDataHeader)) {
             _incomming_header_pos = n;
             return;
@@ -697,13 +620,13 @@ void RedChannel::recive_messages()
                                                                  _incomming_header.type,
                                                                  _incomming_header.size,
                                                                  _incomming_header.sub_list));
-        n = RedPeer::recive((*message)->data(), (*message)->compound_size());
+        n = RedPeer::receive((*message)->data(), (*message)->compound_size());
         if (n != (*message)->compound_size()) {
             _incomming_message = message.release();
             _incomming_message_pos = n;
             return;
         }
-        on_message_recived();
+        on_message_received();
         _message_handler->handle_message(*(*message));
         on_message_complition((*message)->serial());
     }
@@ -723,7 +646,7 @@ void RedChannel::on_event()
     send_messages();
 
     if (_incomming_header_pos) {
-        _incomming_header_pos += RedPeer::recive(((uint8_t*)&_incomming_header) +
+        _incomming_header_pos += RedPeer::receive(((uint8_t*)&_incomming_header) +
                                                  _incomming_header_pos,
                                                  sizeof(SpiceDataHeader) - _incomming_header_pos);
         if (_incomming_header_pos != sizeof(SpiceDataHeader)) {
@@ -738,7 +661,7 @@ void RedChannel::on_event()
     }
 
     if (_incomming_message) {
-        _incomming_message_pos += RedPeer::recive(_incomming_message->data() +
+        _incomming_message_pos += RedPeer::receive(_incomming_message->data() +
                                                   _incomming_message_pos,
                                                   _incomming_message->compound_size() -
                                                   _incomming_message_pos);
@@ -747,11 +670,11 @@ void RedChannel::on_event()
         }
         AutoRef<CompoundInMessage> message(_incomming_message);
         _incomming_message = NULL;
-        on_message_recived();
+        on_message_received();
         _message_handler->handle_message(*(*message));
         on_message_complition((*message)->serial());
     }
-    recive_messages();
+    receive_messages();
 }
 
 void RedChannel::send_migrate_flush_mark()
@@ -786,7 +709,7 @@ void RedChannel::handle_migrate(RedPeer::InMessage* message)
     }
     AutoRef<CompoundInMessage> data_message;
     if (migrate->flags & SPICE_MIGRATE_NEED_DATA_TRANSFER) {
-        data_message.reset(recive());
+        data_message.reset(receive());
     }
     _client.migrate_channel(*this);
     if (migrate->flags & SPICE_MIGRATE_NEED_DATA_TRANSFER) {
