@@ -66,8 +66,6 @@ public:
      uint32_t get_peer_major() { return _remote_major;}
      uint32_t get_peer_minor() { return _remote_minor;}
 
-     virtual void swap(RedChannelBase* other);
-
 protected:
     void set_common_capability(uint32_t cap);
     void set_capability(uint32_t cap);
@@ -108,16 +106,6 @@ public:
     virtual void on_event();
 };
 
-class MigrationDisconnectSrcEvent: public Event {
-public:
-    virtual void response(AbstractProcessLoop& events_loop);
-};
-
-class MigrationConnectTargetEvent: public Event {
-public:
-    virtual void response(AbstractProcessLoop& events_loop);
-};
-
 struct SyncInfo {
     Mutex* lock;
     Condition* condition;
@@ -138,10 +126,7 @@ public:
     virtual void disconnect();
     virtual bool abort();
 
-    virtual void disconnect_migration_src();
-    virtual void connect_migration_target();
-
-    virtual CompoundInMessage *recive();
+    virtual CompoundInMessage *receive();
 
     virtual void post_message(RedChannel::OutMessage* message);
     int get_connection_error() { return _error;}
@@ -155,8 +140,6 @@ protected:
     virtual void on_connect() {}
     virtual void on_disconnect() {}
     virtual void on_migrate() {}
-    virtual void on_disconnect_mig_src() { on_disconnect();}
-    virtual void on_connect_mig_target() { on_connect();}
     void handle_migrate(RedPeer::InMessage* message);
     void handle_set_ack(RedPeer::InMessage* message);
     void handle_ping(RedPeer::InMessage* message);
@@ -171,13 +154,11 @@ private:
     void run();
     void send_migrate_flush_mark();
     void send_messages();
-    void recive_messages();
+    void receive_messages();
     void on_send_trigger();
     virtual void on_event();
-    void on_message_recived();
+    void on_message_received();
     void on_message_complition(uint64_t serial);
-    void do_migration_disconnect_src();
-    void do_migration_connect_target();
 
     static void* worker_main(void *);
 
@@ -222,8 +203,6 @@ private:
     uint64_t _disconnect_reason;
 
     friend class SendTrigger;
-    friend class MigrationDisconnectSrcEvent;
-    friend class MigrationConnectTargetEvent;
 };
 
 
